#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-18 09:26:59
@LastEditors: Li Fajin
@LastEditTime: 2019-09-30 15:11:16
@Description: This file is used for local cAI and global cAI calculation of each gene

Notes:
1) part of codes adapted from CAI package: Lee, B. D. (2018). Python Implementation of Codon Adaptation Index. Journal of Open Source Software, 3 (30), 905. https://doi.org/10.21105/joss.00905
2) input files contains:
	a. interested sequences with fasta format separated by comma.
	b. either reference cds sequences with fasta format.  e.g. some high expressed genes, whose cds sequences could be generated by GetProteinCodingSequence.py
3) output files contains:
	a. if given a reference sequence file, a RUSCs and weights used for CAI calculation would be generated.
	b. a local cAI values at each position would be generated.  One fasta file, one this kind of file.
	c. a meta results at each position would be generated.
	d. a global CAI values of each gene.

'''

from .FunctionDefinition import *
from collections import Counter
from collections import defaultdict
from itertools import chain
from functools import reduce
from operator import mul
import Bio.Data.CodonTable as ct
# get rid of Biopython warning
import warnings
from scipy.stats import gmean
from Bio import BiopythonWarning
warnings.simplefilter("ignore", BiopythonWarning)



def synonymous_codons(genetic_code_dict):
	''' code adapted from CAI'''

	# invert the genetic code dictionary to map each amino acid to its codons
	codons_for_amino_acid = defaultdict(list)
	for codon, amino_acid in genetic_code_dict.items():
		codons_for_amino_acid[amino_acid].append(codon)

	# create dictionary of synonymous codons
	# Example: {'CTT': ['CTT', 'CTG', 'CTA', 'CTC', 'TTA', 'TTG'], 'ATG': ['ATG']...}
	return {
		codon: codons_for_amino_acid[genetic_code_dict[codon]]
		for codon in genetic_code_dict.keys()
	}

def RSCU(sequences, synonymousCodonsDict,genetic_code=1):
	''' code adapted from CAI'''
	if not isinstance(sequences, (list, tuple)):
		raise ValueError(
			"Be sure to pass a list of sequences, not a single sequence. "
			"To find the RSCU of a single sequence, pass it as a one element list."
		)

	# ensure all input sequences are divisible by three
	for sequence in sequences:
		if len(sequence) % 3 != 0:
			print(sequence+" cannot be divided by 3!",file=sys.stderr)
		if not sequence:
			raise ValueError("Input sequence cannot be empty")

	# count the number of each codon in the sequences
	codon_sequences = (
		(sequence[i : i + 3].upper() for i in range(0, len(sequence), 3))
		for sequence in sequences if len(sequence) % 3 == 0
	)
	codons = chain.from_iterable(
		codon_sequences
	)  # flat list of all codons (to be used for counting)
	counts = Counter(codons)
	# "if a certain codon is never used in the reference set... assign [its
	# count] a value of 0.5"
	for codon in ct.unambiguous_dna_by_id[genetic_code].forward_table:
		if counts[codon] == 0:
			counts[codon] = 0.5

	# determine the synonymous codons for the genetic code
	synonymousCodons = synonymousCodonsDict[genetic_code]

	# hold the result as it is being calulated
	result = {}

	# calculate RSCU values
	for codon in ct.unambiguous_dna_by_id[genetic_code].forward_table:
		result[codon] = counts[codon] / (
			(len(synonymousCodons[codon]) ** -1)
			* (sum((counts[_codon] for _codon in synonymousCodons[codon])))
		)


	return result


def relative_adaptiveness(synonymousCodonsDict,sequences=None, RSCUs=None, genetic_code=1):
	''' code adapted from CAI'''

	# ensure user gave only and only one input
	if sum([bool(sequences), bool(RSCUs)]) != 1:
		raise TypeError("Must provide either reference sequences or RSCU dictionary")

	# calculate the RSCUs if only given sequences
	if sequences:
		RSCUs = RSCU(sequences, synonymousCodonsDict,genetic_code=genetic_code)

	# determine the synonymous codons for the genetic code
	synonymousCodons = synonymousCodonsDict[genetic_code]

	# calculate the weights
	weight = {}
	for codon in RSCUs:
		weight[codon] = RSCUs[codon] / max(
			(RSCUs[_codon] for _codon in synonymousCodons[codon])
		)
	for stopCodon in ['TAG','TGA','TAA']:
		weight[stopCodon]=0
	return weight

def CAI_of_each_trans(sequence, synonymousCodonsDict,non_synonymous_codons,weights=None, RSCUs=None, reference=None, genetic_code=1):
	''' codes adapted from CAI'''
	# validate user input
	if sum([bool(reference), bool(RSCUs)], bool(weights)) != 1:
		raise TypeError(
			"Must provide either reference sequences, or RSCU dictionary, or weights"
		)

	# validate sequence
	if not sequence:
		raise ValueError("Sequence cannot be empty")

	# make sure input sequence can be divided into codons. If so, split into list of codons
	if len(sequence) % 3 != 0:
		raise ValueError("Input sequence not divisible by three")
	sequence = sequence.upper()
	sequence = [sequence[i : i + 3] for i in range(0, len(sequence), 3)]

	# generate weights if not given
	if reference:
		weights = relative_adaptiveness(synonymousCodonsDict,sequences=reference, genetic_code=genetic_code)
	elif RSCUs:
		weights = relative_adaptiveness(synonymousCodonsDict,RSCUs=RSCUs, genetic_code=genetic_code)
	sequence_weights = []
	for codon in sequence:
		if codon not in non_synonymous_codons[genetic_code]:
			try:
				sequence_weights.append(weights[codon])
			except KeyError:
				# ignore stop codons
				if codon in ct.unambiguous_dna_by_id[genetic_code].stop_codons:
					pass
				else:
					raise KeyError(
						"Bad weights dictionary passed: missing weight for codon "
						+ str(codon)
						+ "."
					)
	return float(gmean(sequence_weights))

def global_cAI(sequenceDict,synonymousCodonsDict,non_synonymous_codons,weights=None, RSCUs=None, reference=None, genetic_code=1):
	'''calculate global CAI'''
	cAI={}
	for trans in sequenceDict.keys():
		cds_seq=sequenceDict[trans][:-3] ## excluding stop codon
		if len(cds_seq) % 3 != 0:
			continue
		cAI[trans]=CAI_of_each_trans(cds_seq, synonymousCodonsDict,non_synonymous_codons,weights=weights, RSCUs=RSCUs, reference=reference, genetic_code=genetic_code)
	return cAI


def get_trans_frame_cAI(sequenceDict,upLength,downLength,weight,table=1):
	'''get the local cAI in each codon'''
	startcAI=np.zeros(int(upLength+downLength+1),dtype='float64')
	stopcAI=np.zeros(int(upLength+downLength+1),dtype='float64')
	startcAIList=[]
	stopcAIList=[]
	startPos=[]
	stopPos=[]
	cAI={}
	in_selectTrans=sequenceDict.keys()
	# print(weight,file=sys.stderr)
	for trans in in_selectTrans:
		cds_seq=sequenceDict[trans][:-3] # exclude stop codon
		if len(cds_seq) %3 !=0:
			continue
		tmpcAI=[]
		codon_seq=[cds_seq[i:i+3] for i in np.arange(0,len(cds_seq),3)]
		for codon in codon_seq:
			tmpcAI.append(weight[codon])
		(tmpStartWin,tmpStartPos)=getWindowsVector(upLength,downLength,tmpcAI,0) #start codon coor is 0 (0-based), codon level
		(tmpStopWin, tmpStopPos) =getWindowsVector(downLength,upLength,tmpcAI,(len(tmpcAI)-1))  #stop codon coor is len-1 (0-based) codon level
		startcAIList.append(tmpStartWin)
		stopcAIList.append(tmpStopWin)
		startPos.append(tmpStartPos)
		stopPos.append(tmpStopPos)
		cAI[trans]=tmpcAI
	startcAIList=np.array(startcAIList)
	startPos=np.array(startPos)
	stopcAIList=np.array(stopcAIList)
	stopPos=np.array(stopPos)
	for terms in np.arange(upLength+downLength+1):
		startcAI[terms]=np.mean(startcAIList[np.where(startPos[:,terms]==1),terms])
		stopcAI[terms] =np.mean(stopcAIList[np.where(stopPos[:,terms]==1),terms])
	return(startcAI,stopcAI,cAI)

def parse_weight_file(inputFile):
	'''The RSCU file contains two columns: codon and weight'''
	weights={}
	i=0
	with open(inputFile,'r') as f:
		for line in f:
			i+=1
			if line.strip()=="":
				continue
			if i==1:
				continue
			codon=str(line.strip().split("\t")[0])
			weight=float(line.strip().split("\t")[1])
			weights[codon]=weight
	return weights

def write_weight_file(weights,outputFile):
	''' output the RSCUs or weights '''
	with open(outputFile,"w") as f:
		f.write("%s\t%s\n" %("codons","weights"))
		for k,v in weights.items():
			f.write("%s\t%s\n" %(str(k),str(v)))


def write_trans_file_cAI_dataframe(inFastaAttr,outFile):
	data=[]
	for fasta in inFastaAttr:
		k=pd.DataFrame([fasta.fastaLegend]*len(fasta.startcAI))
		start=pd.DataFrame(fasta.startcAI)
		stop=pd.DataFrame(fasta.stopcAI)
		cAI=pd.merge(start,stop,how="left",left_index=True,right_index=True)
		cAI=pd.merge(k,cAI,how="left",left_index=True,right_index=True)
		data.append(cAI)
	temp=data[0]
	if len(data) < 1:
		raise EOFError("Empty file, there is nothing in the file.")
	if len(data) == 1:
		temp.columns=["sample","start_cAI","stop_cAI"]
		temp.to_csv(outFile,sep="\t",index=0)
	else:
		for i in np.arange(1,len(data)):
			temp=np.vstack((temp,data[i]))
		temp=pd.DataFrame(temp,columns=["sample","start_cAI","stop_cAI"])
		temp.to_csv(outFile,sep="\t",index=0)

def write_cAI_of_each_gene(inFastaAttr,outFile):
	data=[]
	data_index=[]
	for fasta in inFastaAttr:
		d=fasta.cAI
		i=fasta.fastaLegend
		data.append(d)
		data_index.append(i)
	data=pd.DataFrame(data,index=data_index)
	data=data.T
	data.to_csv(outFile,sep="\t")

def write_cAI_per_codon(inFastaAttr,outFile):
	for fasta in inFastaAttr:
		with open(outFile+"_"+fasta.fastaLegend+"_local_cAI_each_position.txt",'w') as f:
			f.write("%s\t%s\n" %("transcripts","local_cAI"))
			for trans,cAIperCodons in fasta.cAIPerCodon.items():
				f.write("%s\t" %(trans))
				for pos in range(len(cAIperCodons)):
					f.write("%s\t" %(str(cAIperCodons[pos])))
				f.write("\n")


def main():
	parser=create_parser_for_cAI()
	(options,args)=parser.parse_args()
	synonymousCodonsDict = {k: synonymous_codons(v.forward_table) for k, v in ct.unambiguous_dna_by_id.items()}
	non_synonymous_codons = {k: {codon for codon in v.keys() if len(v[codon]) == 1}for k, v in synonymousCodonsDict.items()}
	# validate user input
	if sum([bool(options.reference), bool(options.RSCUs)], bool(options.weights)) != 1:
		raise TypeError(
			"Must provide either reference sequences, or RSCU dictionary, or weights"
		)
	if not options.transcriptFiles:
		raise IOError("Please input your interested sequences with fasta format!")
	transcriptFiles=options.transcriptFiles.strip().split(",")
	transFileLegend=options.trans_file_legend.strip().split(",")
	references=options.reference
	RSCUs=options.RSCUs
	weights=options.weights
	## handle bam file attr
	fasta_attr=[]
	for ii,jj in zip(transcriptFiles,transFileLegend):
		fasta=fasta_attrbution(ii,jj)
		fasta_attr.append(fasta)
	print("your input : "+ str(len(transcriptFiles))+" transcript files",file=sys.stderr)
	if references:
		referenceDict=fastaIter(references)
		references=list(referenceDict.values())
		RSCUs=RSCU(references,synonymousCodonsDict,genetic_code=options.genetic_table)
		weights = relative_adaptiveness(synonymousCodonsDict,RSCUs=RSCUs, genetic_code=options.genetic_table)
		write_weight_file(weights,options.output_prefix+"_weights_for_cAI.txt")
		write_weight_file(RSCUs,options.output_prefix+"_RSCUs_for_cAI.txt")
		for fasta in fasta_attr:
			sequenceDict=fastaIter(fasta.fastaName)
			(fasta.startcAI,fasta.stopcAI,fasta.cAIPerCodon) = get_trans_frame_cAI(sequenceDict,options.upstream_codon,options.downstream_codon,weights,table=options.genetic_table)
			fasta.cAI=global_cAI(sequenceDict,synonymousCodonsDict,non_synonymous_codons,weights=weights,genetic_code=options.genetic_table)
	elif RSCUs:
		RSCUs=parse_weight_file(RSCUs)
		weights = relative_adaptiveness(synonymousCodonsDict,RSCUs=RSCUs, genetic_code=options.genetic_table)
		write_weight_file(weights,options.output_prefix+"_weights_for_cAI.txt")
		for fasta in fasta_attr:
			sequenceDict=fastaIter(fasta.fastaName)
			(fasta.startcAI,fasta.stopcAI,fasta.cAIPerCodon) = get_trans_frame_cAI(sequenceDict,options.upstream_codon,options.downstream_codon,weights,table=options.genetic_table)
			fasta.cAI=global_cAI(sequenceDict,synonymousCodonsDict,non_synonymous_codons,RSCUs=RSCUs,genetic_code=options.genetic_table)
	elif weights:
		weights=parse_weight_file(weights)
		for fasta in fasta_attr:
			sequenceDict=fastaIter(fasta.fastaName)
			(fasta.startcAI,fasta.stopcAI,fasta.cAIPerCodon) = get_trans_frame_cAI(sequenceDict,options.upstream_codon,options.downstream_codon,weights,table=options.genetic_table)
			fasta.cAI=global_cAI(sequenceDict,synonymousCodonsDict,non_synonymous_codons,weights=weights,genetic_code=options.genetic_table)
	else:
		raise IOError("Please enter a correct input. Must provide either reference sequences, or RSCU dictionary, or weights")


	print("Finish the step of get_trans_frame_cAI...",file=sys.stderr)
	write_trans_file_cAI_dataframe(fasta_attr,options.output_prefix+"_local_cAI_dataframe.txt")
	write_cAI_of_each_gene(fasta_attr,options.output_prefix+"_global_cAI.txt")
	write_cAI_per_codon(fasta_attr,options.output_prefix)
	print("Finish the step of cAI calculation.",file=sys.stderr)

if  __name__=="__main__":
	main()



#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-15 21:29:12
@LastEditors: Li Fajin
@LastEditTime: 2019-09-18 20:38:23
@Description: This script is used for checking periodicity of ribosome profiling data, but without P-site identification.
And the part code are adapted from RiboCode our lab developed before. [Xiao, et al. NAR.2018]
usage: python Periodicity -i bam -a RiboCode_annote -c longest.trans.info.txt -o outprefix -L 25 -R 35 --id-type transcript-id
'''


from .FunctionDefinition import *
from functools import reduce
import pickle
from collections import defaultdict
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import seaborn as sns
from matplotlib.backends.backend_pdf import PdfPages
from optparse import OptionParser


def load_transcripts_pickle(pickle_file):
	if os.path.exists(pickle_file):
		sys.stdout.write("\tLoading transcripts.pickle ...\n")
		with open(pickle_file,"rb") as fin:
			gene_dict, transcript_dict = pickle.load(fin)
	else:
		raise IOError("\tError, %s file not found\n" % pickle_file)

	return gene_dict,transcript_dict

def periodicity(in_bamFile,in_selectTrans,transcript_dict,left_length,right_length):
	pysamFile=pysam.AlignmentFile(in_bamFile,"rb")
	pysamFile_trans=pysamFile.references
	in_selectTrans=set(pysamFile_trans).intersection(in_selectTrans)
	start_density=defaultdict(lambda:np.zeros(101,dtype="int64"))
	stop_density=defaultdict(lambda:np.zeros(101,dtype="int64"))
	total_reads=0
	specific_counts=defaultdict(int)
	i=0
	for trans in in_selectTrans:
		i+=1
		for record in pysamFile.fetch(trans):
			if record.flag == 16 or record.flag == 272:
				continue
			total_reads += 1
			specific_counts[record.query_length]+=1
			if trans not in transcript_dict.keys():
				continue
			if left_length<=record.query_length<=right_length:
				distance_to_start=record.reference_start-transcript_dict[trans].startcodon.start
				dsitance_to_stop=record.reference_start-transcript_dict[trans].stopcodon.end
				if abs(distance_to_start)<=50:
					start_density[record.query_length][50+distance_to_start]+=1
				if abs(dsitance_to_stop) <= 50:
					stop_density[record.query_length][50+dsitance_to_stop]+=1

	specific_counts['sum']=total_reads
	print("There are " + str(i)+" used for statistics.",file=sys.stderr)
	return start_density,stop_density,total_reads,specific_counts

def plot_periodicity_start_codon(start_density,specific_counts,output_prefix):
	start_density=pd.DataFrame(start_density)
	start_density=start_density.reindex(columns=sorted(start_density.columns))
	start_density['sum']=start_density.apply(sum,axis=1)
	start_density.loc['sum']=np.array([specific_counts[key] for key in start_density.columns])
	start_density.to_csv(output_prefix+"_from_start_codon.txt",index=0,sep="\t")
	text_font={"size":15,"family":"Arial","weight":"bold"}
	plt.rc('font',weight='bold')
	with PdfPages(output_prefix + "_start_periodicity.pdf") as pdf:
		x = np.arange(-50,51,dtype=int)
		colors = sns.color_palette('husl',3)*34
		for L in start_density.columns:
			xticks=[-40,-20,0,20,40]
			perct = '{:.2%}'.format(specific_counts[L] / specific_counts['sum'])
			fig,ax1 = plt.subplots(figsize=(5.6,2.8))
			y1=start_density.loc[:,L]
			ax1.vlines(x,ymin=np.zeros(101),ymax=y1,colors=colors[:-1],linewidth=2)
			ax1.tick_params(axis='x',which="both",top=False,direction='out')
			ax1.set_xticks(xticks)
			ax1.set_xlim((-50,50))
			ax1.spines["top"].set_visible(False)
			ax1.spines["right"].set_visible(False)
			ax1.spines["bottom"].set_linewidth(2)
			ax1.spines["left"].set_linewidth(2)
			ax1.set_xlabel("Distance from start codon (nt)",fontdict=text_font)
			ax1.set_ylabel("Alignments",fontdict=text_font)
			if not L == 'sum':
				ax1.set_title("{} nt reads,proportion:{}".format(L,perct),fontdict=text_font)
			else:
				ax1.set_title("Total reads, proporation:{}".format(perct),fontdict=text_font)
			fig.tight_layout()
			pdf.savefig(fig)
			plt.close()
	return None
def plot_periodicity_stop_codon(stop_density,specific_counts,output_prefix):
	stop_density=pd.DataFrame(stop_density)
	stop_density=stop_density.reindex(columns=sorted(stop_density.columns))
	stop_density['sum']=stop_density.apply(sum,axis=1)
	stop_density.loc['sum']=np.array([specific_counts[key] for key in stop_density.columns])
	stop_density.to_csv(output_prefix+"_from_stop_codon.txt",index=0,sep="\t")
	text_font={"size":15,"family":"Arial","weight":"bold"}
	plt.rc('font',weight='bold')
	with PdfPages(output_prefix + "_stop_periodicity.pdf") as pdf:
		x = np.arange(-50,51,dtype=int)
		colors = sns.color_palette('husl',3)*34
		for L in stop_density.columns:
			xticks=[-40,-20,0,20,40]
			perct = '{:.2%}'.format(specific_counts[L] / specific_counts['sum'])
			fig,ax1 = plt.subplots(figsize=(5.6,2.8))
			y1=stop_density.loc[:,L]
			ax1.vlines(x,ymin=np.zeros(101),ymax=y1,colors=colors[:-1],linewidth=2)
			ax1.tick_params(axis='x',which="both",top=False,direction='out')
			ax1.set_xticks(xticks)
			ax1.set_xlim((-50,50))
			ax1.spines["top"].set_visible(False)
			ax1.spines["right"].set_visible(False)
			ax1.spines["bottom"].set_linewidth(2)
			ax1.spines["left"].set_linewidth(2)
			ax1.set_xlabel("Distance from stop codon (nt)",fontdict=text_font)
			ax1.set_ylabel("Alignments",fontdict=text_font)
			if not L == 'sum':
				ax1.set_title("{} nt reads,proportion:{}".format(L,perct),fontdict=text_font)
			else:
				ax1.set_title("Total reads, proporation:{}".format(perct),fontdict=text_font)
			fig.tight_layout()
			pdf.savefig(fig)
			plt.close()
	return None

def main():
	parsed=create_parser_for_periodicity()
	(options,args)=parsed.parse_args()
	## calculate density for each bam files
	if not options.annot_dir:
		raise IOError("Please run RiboCode::prepare_transcripts first")
	else:
		gene_dict,transcript_dict = load_transcripts_pickle(os.path.join(options.annot_dir,"transcripts.pickle"))
	selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict=reload_transcripts_information(options.coorFile)
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if options.in_selectTrans:
		select_trans=pd.read_csv(options.in_selectTrans,sep="\t")
		select_trans=set(select_trans.iloc[:,0].values)
		if options.id_type == 'transcript_id':
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans)) + " transcripts from "+options.in_selectTrans+" used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_id':
			tmp=[geneID2transID[gene_id] for gene_id in select_trans]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene id could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_name' or options.id_type=='gene_symbol':
			tmp=[geneName2transID[gene_name] for gene_name in select_trans]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene symbol could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	else:
		select_trans=selectTrans
	print("There are "+str(len(select_trans))+" transcripts with both start and stop codon will be used for following analysis.",file=sys.stderr)
	print("Start calculating read density...",file=sys.stderr)
	start_density,stop_density,total_reads,specific_counts=periodicity(options.bamFile,select_trans,transcript_dict,options.left_length,options.right_length)
	plot_periodicity_start_codon(start_density,specific_counts,options.output_prefix)
	plot_periodicity_stop_codon(stop_density,specific_counts,options.output_prefix)
	print("Finish the step of plot periodicity",file=sys.stderr)


if __name__=="__main__":
	main()


#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-12 19:56:24
@LastEditors: Li Fajin
@LastEditTime: 2019-09-28 17:38:15
@Description: the user could directly input the dataframe format file generated by MetageneAnalysis.py and plot the ribosome footprint density without re-do from the bam files
			1) the input file must be python DataFrame format, and has three required columns:
				1. columns one : sample name [required].
				2. columns two : footprint density from the start codon [required].
				3. columns three: footprint density from the stop codon [required].
				4. start_lower_CI: lower confidence value of density from start codon.
				5. start_higher_CI: higher confidence value of density from start codon.
				6. stop_lower_CI: lower confidence value of density from stop codon.
				7. stop_highrt_CI: higher confidence value of density from stop codon.
			2) the output file could be pdf/png/jpg format
'''

import numpy as np
import pandas as pd
import sys
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import seaborn as sns
from optparse import OptionParser
from functools import reduce
from itertools import chain
from collections import defaultdict
from .__init__ import __version__

def create_parser_for_metagene_plot():
	'''argument parser'''
	usage="usage: python %prog [options]" +"\n" + __doc__+"\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input",action="store",type="string",dest="density_file",help="Input file in txt format.And the files has three columns; column 1: sample;columns 2: start_density; column 3: stop_density")
	parser.add_option("-d","--downstream_codon",action="store",type="int",default=500, dest="downstream_codon", help="Downstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the upstream codon.")
	parser.add_option("-u","--upstream_codon",action="store",type="int",default=0,dest="upstream_codon",help="Upstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the downstream codon.")
	parser.add_option("-o","--output",action="store",type="string",dest="output_prefix",help="Prefix of output files.[required]")
	parser.add_option('-g','--group',action="store",type="string",dest="group_name",help="Group name of each group separated by comma. e.g. 'si-control,si-eIF3e'")
	parser.add_option('-r','--replicate',action="store",type="string",dest="replicate_name",help="Replicate name of each group separated by comma. e.g. 'si_3e_1_80S,si_3e_2_80S__si_cttl_1_80S,si_ctrl_2_80S'")
	parser.add_option("-f","--format",action="store",type="string",dest="output_format",default='pdf',help="Output file format,'pdf','png' or 'jpg'. default=%default")
	parser.add_option("--ymax",action="store",type="float",dest="ymax",default=None,help="The max of ylim. default=%default")
	parser.add_option("--ymin",action="store",type="float",dest="ymin",default=None,help="The min of ylim. default=%default")
	parser.add_option("-U","--unit_type",action="store",type="string",dest="unit", default="codon",help="Unit type we used for metagene analysis. Either 'nt' or 'codon'. default=%default")
	parser.add_option("--slide-window",action="store",type="string",dest="slideWindow",default=None,help="Using slide window to average the density.Input a  true strings such as yes, y or 1. %default=default")
	parser.add_option("--axvline",action="store",type="float",dest="axvline",default=None,help="Position to plot vetical line")
	parser.add_option("--start",action="store",type="int",dest="start_position",default=5,help="The start position need to be averaged.default=%default")
	parser.add_option("--window",action="store",type="int",dest="window",default=7,help="The length of silde window. ddefault=%default")
	parser.add_option("--step",action="store",type='int',dest="step",default=1,help="The step length of slide window. default=%default")
	parser.add_option("--CI",action="store",type='float',dest="CI",default=None,help="plot the confidence intervals or not. If yes, plot the CI region(95% CI default the same as metageneAnalysis.py). else, no. default=%default")
	parser.add_option("--mode",action="store",type="string",dest="mode",default='all',help="plot all samples or just mean samples [all or mean].If choose 'all',output all samples as well as mean samples, else just mean samples.default=%default")

	return parser

def plot_density_codon_level(data,samples,Type,in_regionLengthParma,in_extendRegionLengthParma,inOutPrefix,inOutFomat,ymin,ymax,unit,axvline,confidence,text_font={"size":20,"family":"Arial","weight":"bold"},legend_font={"size":20,"family":"Arial","weight":"bold"}):
	'''plot the density dsitribution'''
	plt.rc('font',weight='bold')
	fig=plt.figure(figsize=(16,8))
	ax=fig.add_subplot(111)
	winLen=in_regionLengthParma+in_extendRegionLengthParma+1
	# font={"size":20,"family":"Arial","weight":"bold"}
	# colors="bgrcmykwbgrcmykw"
	if len(samples) <=8:
		colors=["b","orangered","green","c","m","y","k","w"]
	else:
		colors=sns.color_palette('husl',len(samples))
	for i in np.arange(len(samples)):
		if Type=="start codon":
			plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,1],color=colors[i],label=samples[i],linewidth=1)
			if confidence:
				ax.fill_between(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,3],data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,4],color=colors[i],alpha=0.2)
			else:
				pass
			ax.set_xticks(np.arange(0,winLen,50))
			ax.set_xticklabels((np.arange(0,winLen,50)-in_extendRegionLengthParma))
			if axvline:
				ax.axvline(axvline,color="r",dashes=(3,2),alpha=0.5)
			else:
				pass

		else:
			plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,2],color=colors[i],label=samples[i],linewidth=1)
			if confidence:
				ax.fill_between(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,5],data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,6],color=colors[i],alpha=0.2)
			else:
				pass
			ax.set_xticks(np.arange(0,winLen,50))
			ax.set_xticklabels((np.arange(0,winLen,50)-in_regionLengthParma))
			if axvline:
				ax.axvline(winLen-axvline,color="r",dashes=(3,2),alpha=0.5)
			else:
				pass

	if unit == 'codon':
		ax.set_xlabel("Distance from "+Type+' (codon)',fontdict=text_font)
	elif unit == 'nt':
		ax.set_xlabel("Distance from "+Type+' (nt)',fontdict=text_font)
	else:
		raise KeyError("KeyError:"+str(unit))
	ax.set_ylabel("Relative footprint density (AU)",fontdict=text_font)
	ax.set_title("Ribosome footprint density profiles",fontdict=text_font)
	ax.spines["top"].set_visible(False)
	ax.spines["right"].set_visible(False)
	ax.spines["bottom"].set_linewidth(2)
	ax.spines["left"].set_linewidth(2)
	ax.tick_params(which="both",width=2,labelsize=20)
	if not ymin and not ymax:
		pass
	elif not ymin and ymax:
		ax.set_ylim(0,ymax)
	elif ymin and not ymax:
		raise IOError("Please offer the ymax parameter as well!")
	elif ymin and ymax:
		ax.set_ylim(ymin,ymax)
	else:
		raise IOError("Please enter correct ymin and ymax parameters!")
	ax.axhline(1,color="r",dashes=(2,3),clip_on=False,alpha=0.5)
	plt.legend(loc="best",prop=legend_font)
	plt.tight_layout()
	plt.savefig(inOutPrefix+"_"+Type+"."+inOutFomat,format=inOutFomat)
	plt.close()

def slide_window_average(data,samples,in_regionLengthParma,in_extendRegionLengthParma,inOutPrefix,start,window,step):
	''' Used for calculating mean density with a slide window'''
	data_average=defaultdict(dict)
	winLen=in_regionLengthParma+in_extendRegionLengthParma+1
	columns=data.columns
	flag=0
	for column in columns:
		data_average[column]=[]
		for i in np.arange(len(samples)):
			if flag == 0:
				data_average[column].extend([samples[i]]*winLen)
			else:
				tmp_data=np.zeros(winLen)
				tmp_data[0:int(start)]+=data.iloc[np.where(data.iloc[:,0]==samples[i])].loc[:,column][0:int(start)]
				tmp_data[-int(start):]+=data.iloc[np.where(data.iloc[:,0]==samples[i])].loc[:,column][-int(start):]
				for j in np.arange(start,winLen-start,step):
					tmp_data[j]+=np.mean(data.iloc[np.where(data.iloc[:,0]==samples[i])].loc[:,column][(j-int((window-1)/2)):(j+int((window-1)/2))])
				data_average[column].extend(tmp_data)
		flag+=1
	data_average=pd.DataFrame(data_average)
	data_average.to_csv(inOutPrefix+"_average_denisty.txt",sep="\t",index=0)
	return data_average

def calculate_mean_data(data,labels_dict,group_names,output_prefix):
	'''calculate mean value'''
	data_dict=defaultdict(list)
	data_mean_dict=defaultdict(dict)
	columns=data.columns.values
	for g in group_names:
		for r in labels_dict[g]:
			data_dict[g].append(data.iloc[np.where(data.iloc[:,0]==r)])

	for g in group_names:
		flag=0
		data_mean_dict[g]={}
		if len(labels_dict[g]) <1:
			raise IOError("Please reset your -g -r parameters because nothing present here.")
		elif len(labels_dict[g]) ==1:
			for column in columns:
				if flag == 0:
					data_mean_dict[g][column]=np.array([g]*data_dict[g][0].shape[0])
					flag+=1
				else:
					data_mean_dict[g][column]=list(reduce(zip,[data_dict[g][i].loc[:,column].values for i in np.arange(len(data_dict[g]))]))

		elif len(labels_dict[g])==2:
			for column in columns:
				if flag == 0:
					data_mean_dict[g][column]=np.array([g]*data_dict[g][0].shape[0])
					flag+=1
				else:
					data_mean_dict[g][column]=np.array([sum(list(chain(i))) for i in list(reduce(zip,[data_dict[g][i].loc[:,column].values for i in np.arange(len(data_dict[g]))]))])/len(labels_dict[g])

		else:
			for column in columns:
				if flag==0:
					data_mean_dict[g][column]=np.array([g]*data_dict[g][0].shape[0])
					flag+=1
				else:
					data_mean_dict[g][column]=np.array([sum(list(flatten(i))) for i in list(reduce(zip,[data_dict[g][i].loc[:,column].values for i in np.arange(len(data_dict[g]))]))])/len(labels_dict[g])


	for k,v in data_mean_dict.items():
		data_mean_dict[k]=pd.DataFrame(v)

	data_mean=pd.concat([v for v in data_mean_dict.values()])
	## write the mean density file
	data_mean.to_csv(output_prefix+"_mean_dataframe.txt",sep="\t",index=0)
	return data_mean

def flatten(xs):
	for x in xs:
		if isinstance(x,tuple):
			for xx in flatten(x):
				yield xx
		else:
			yield x

def parse_plot_density_for_CDS_metagene():
	parsed=create_parser_for_metagene_plot()
	(options,args)=parsed.parse_args()
	(data,in_regionLengthParma,in_extendRegionLengthParma,output_prefix,group_names,replicate_names,output_format,ymin,ymax,unit,axvline,start,window,step,slideWindow,CI,mode)=(options.density_file,
	options.downstream_codon,options.upstream_codon,options.output_prefix,options.group_name.strip().split(','),options.replicate_name.strip().split('__'),
	options.output_format,options.ymin,options.ymax,options.unit,options.axvline,options.start_position,options.window,options.step,options.slideWindow,options.CI,options.mode)
	if window%2 == 0:
		raise IOError("Please reset your --window parameter. It must be a odd number.")
	if (start-1) < (window-1)/2:
		raise IOError("Please reset your --step and --window parameters. The (window-1)/2 must be less than start-1")
	print("your input file is: "+str(data),file=sys.stderr)
	data=pd.read_csv(data,sep="\t")
	samples=np.unique(data.iloc[:,0])
	text_font={"size":30,"family":"Arial","weight":"bold"}
	legend_font={"size":30,"family":"Arial","weight":"bold"}
	text_font_mean={"size":30,"family":"Arial","weight":"bold"}
	legend_font_mean={"size":30,"family":"Arial","weight":"bold"}
	labels_dict={}
	for g,rep in zip(group_names,replicate_names):
		labels_dict[g]=rep.strip().split(',')
	##
	if len(samples) < 1:
		raise IOError("There is no samples in the file, please check your input!")
	if len(samples) == 1:

		plot_density_codon_level(data,samples,"start codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,ymin,ymax,unit,axvline,CI,text_font=text_font,legend_font=legend_font)
		plot_density_codon_level(data,samples,"stop codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,ymin,ymax,unit,axvline,CI,text_font=text_font,legend_font=legend_font)
		print("finished plot the ribosome footprint density",file=sys.stderr)

	## calculate the mean density
	if len(samples) >1 :
		data_mean=calculate_mean_data(data,labels_dict,group_names,output_prefix)
		samples_new=np.unique(data_mean.iloc[:,0])
		if slideWindow:
		## slide average
			data_average=slide_window_average(data,samples,in_regionLengthParma,in_extendRegionLengthParma,output_prefix,start,window,step)
			data_mean_average=slide_window_average(data_mean,samples_new,in_regionLengthParma,in_extendRegionLengthParma,output_prefix+"_mean",start,window,step)
			if mode in ['all','All','a','A']:
				plot_density_codon_level(data_average,samples,"start codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix+"_average",output_format,ymin,ymax,unit,axvline,CI,text_font=text_font,legend_font=legend_font)
				plot_density_codon_level(data_average,samples,"stop codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix+"_average",output_format,ymin,ymax,unit,axvline,CI,text_font=text_font,legend_font=legend_font)
			elif mode in ['mean','Mean','m','M']:
				plot_density_codon_level(data_mean_average,samples_new,"start codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix+"_mean_average",output_format,ymin,ymax,unit,axvline,CI,text_font=text_font_mean,legend_font=legend_font_mean)
				plot_density_codon_level(data_mean_average,samples_new,"stop codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix+"_mean_average",output_format,ymin,ymax,unit,axvline,CI,text_font=text_font_mean,legend_font=legend_font_mean)
			else:
				raise IOError("Please enter a correct --mode parameter [all or mean]")
		else:

			## plot density
			if mode in ['all','All','a','A']:
				plot_density_codon_level(data,samples,"start codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,ymin,ymax,unit,axvline,CI,text_font=text_font,legend_font=legend_font)
				plot_density_codon_level(data,samples,"stop codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,ymin,ymax,unit,axvline,CI,text_font=text_font,legend_font=legend_font)
			elif mode in ['mean','Mean','m','M']:
				plot_density_codon_level(data_mean,samples_new,"start codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix+"_mean",output_format,ymin,ymax,unit,axvline,CI,text_font=text_font_mean,legend_font=legend_font_mean)
				plot_density_codon_level(data_mean,samples_new,"stop codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix+"_mean",output_format,ymin,ymax,unit,axvline,CI,text_font=text_font_mean,legend_font=legend_font_mean)
			else:
				raise IOError("Please enter a correct --mode parameter [all or mean]")
			# plot_density_comparison(data_mean,samples_new,"stop codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix+"_mean",output_format,ymax,unit,text_font=text_font_mean,legend_font=legend_font_mean)
		print("finished plot the ribosome footprint density",file=sys.stderr)


def main():
	parse_plot_density_for_CDS_metagene()

if __name__ =="__main__":
	main()



#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-28 15:07:53
@LastEditors: Li Fajin
@LastEditTime: 2019-08-30 16:34:02
@Description: This script is used for calculating mean ribosome density of each position for each transcript.
usage: python enrichmentMeanDensity.py -i rep1.txt,rep2.txt,rep3.txt -o outprefix
'''

import sys
import numpy as np
from functools import reduce
from collections import defaultdict
from optparse import OptionParser
from .__init__ import __version__

def create_parse_for_mean_enrichment():
	'''argument parser.'''
	usage="usage: %prog [options]" + '\n' + __doc__ + "\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input",action="store",type="string",dest="inputFiles",help="Ribosome density files of different replicates separated by comma which could be generated by RiboDensityAtEachPosition.py. e.g. rep1.txt,rep2.txt")
	parser.add_option("-o","--output",action="store",type="string",dest="outprefix",help="Output prefix of the output file.")
	return parser
def get_density_dict(densityFile):
	density_dict={}
	with open(densityFile,'r') as f:
		i=0
		for line in f:
			i+=1
			if i==1:
				continue
			tmp=line.strip().split("\t")
			trans_id=tmp[0]
			density_dict[trans_id]=[float(i) for i in tmp[1:]]
	return density_dict

def flatten(xs):
	for x in xs:
		if isinstance(x,tuple):
			for xx in flatten(x):
				yield xx
		else:
			yield x

def meanDensity(inputFiles,outprefix):
	all_ratio_dict=defaultdict(list)
	for f in inputFiles:
		ratio_dict=get_density_dict(f)
		for trans,ratio in ratio_dict.items():
			all_ratio_dict[trans].append(ratio)
	with open(outprefix+"_mean_density.txt",'w') as fout:
		fout.write("%s\t%s\n" %("transcript","mean_density"))
		for trans,ratio in all_ratio_dict.items():
			mean_ratio=np.array(list(map(np.mean,[list(flatten(i)) for i in list(reduce(zip,all_ratio_dict[trans]))])))
			fout.write("%s\t" %(trans))
			for i in range(len(mean_ratio)):
				fout.write("%s\t" %str(mean_ratio[i]))
			fout.write("\n")

def main():
	parser=create_parse_for_mean_enrichment()
	(options,args)=parser.parse_args()
	if not options.inputFiles or not options.outprefix:
		raise IOError("Please set your -i and -o parameters!")
	inputFiles=options.inputFiles.strip().split(",")
	print("Start calculating mean density...",file=sys.stderr)
	meanDensity(inputFiles,options.outprefix)
	print("Finish the step of mean density calculation!",file=sys.stderr)

if __name__=="__main__":
	main()




#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-28 14:02:54
@LastEditors: Li Fajin
@LastEditTime: 2019-08-30 16:42:02
@Description: This script is used for plot of enrichment ratio.
1) the input file must be python DataFrame format, and has required three columns:
		1. columns one : sample name
		2. columns two : enrichment ratio values from the start codon
		3. columns three: enrichment ratio values from the stop codon
2) the output file could be pdf/png/jpg format
'''

import numpy as np
import pandas as pd
import sys
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import seaborn as sns
from optparse import OptionParser
from functools import reduce
from itertools import chain
from collections import defaultdict
from .__init__ import __version__

def create_parser():
	'''argument parser'''
	usage="usage: python %prog [options]" +"\n" + __doc__+"\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input",action="store",type="string",dest="density_file",help="Input file in txt format.And the files has three columns; column 1: sample;columns 2: start_density; column 3: stop_density")
	parser.add_option("-d","--downstream_codon",action="store",type="int",default=500, dest="downstream_codon", help="Downstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the upstream codon.")
	parser.add_option("-u","--upstream_codon",action="store",type="int",default=0,dest="upstream_codon",help="Upstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the downstream codon.")
	parser.add_option("-o","--output",action="store",type="string",dest="output_prefix",help="Prefix of output files.[required]")
	parser.add_option("-f","--format",action="store",type="string",dest="output_format",default='pdf',help="Output file format,'pdf','png' or 'jpg'. default=%default")
	parser.add_option("--ymin",action="store",type="float",dest="ymin",default=None,help="The max of ylim. default=%default")
	parser.add_option("--ymax",action="store",type="float",dest="ymax",default=None,help="The max of ylim. default=%default")
	parser.add_option("--unit",action="store",type="string",dest="unit",default='codon',help="Unit for density calculation.[codon or nt]")
	parser.add_option("--mode",action="store",type="string",dest="mode",default='all',help="Mode for plot, you can chose plot all samples or single sample. [all or single]")
	parser.add_option("--axvline",action="store",type="float",dest="axvline",default=None,help="Position to plot a vertical line in x axis. default=%default")
	parser.add_option("--axhline",action="store",type="float",dest="axhline",default=None,help="Position to plot a vertical line in y axis. default=%default")
	parser.add_option("--label",action="store",type='string',dest='label_list',default=None,help="The legend labels used for plot. default=%default")
	parser.add_option("--slide-window",action="store",type="string",dest="slideWindow",default=None,help="Using slide window to average the density.Input a	true strings such as yes, y or 1. %default=default")
	parser.add_option("--start",action="store",type="int",dest="start_position",default=5,help="The start position need to be averaged.default=%default")
	parser.add_option("--window",action="store",type="int",dest="window",default=7,help="The length of silde window. ddefault=%default")
	parser.add_option("--step",action="store",type='int',dest="step",default=1,help="The step length of slide window. default=%default")
	parser.add_option("--CI",action="store",type='float',dest="CI",default=None,help="plot the confidence intervals or not. If yes, plot the CI region(95% CI default the same as metageneAnalysis.py). else, no. default=%default")


	return parser

def slide_window_average(data,samples,in_regionLengthParma,in_extendRegionLengthParma,inOutPrefix,start,window,step):
	''' Used for calculating mean density with a slide window'''
	data_average=defaultdict(dict)
	winLen=in_regionLengthParma+in_extendRegionLengthParma+1
	columns=data.columns
	flag=0
	for column in columns:
		data_average[column]=[]
		for i in np.arange(len(samples)):
			if flag == 0:
				data_average[column].extend([samples[i]]*winLen)
			else:
				tmp_data=np.zeros(winLen)
				tmp_data[0:int(start)]+=data.iloc[np.where(data.iloc[:,0]==samples[i])].loc[:,column][0:int(start)]
				tmp_data[-int(start):]+=data.iloc[np.where(data.iloc[:,0]==samples[i])].loc[:,column][-int(start):]
				for j in np.arange(start,winLen-start,step):
					tmp_data[j]+=np.mean(data.iloc[np.where(data.iloc[:,0]==samples[i])].loc[:,column][(j-int((window-1)/2)):(j+int((window-1)/2))])
				data_average[column].extend(tmp_data)
		flag+=1
	data_average=pd.DataFrame(data_average)
	data_average.to_csv(inOutPrefix+"_average_denisty.txt",sep="\t",index=0)
	return data_average

def plot_density_for_all_samples(data,samples,Type,in_regionLengthParma,in_extendRegionLengthParma,inOutPrefix,inOutFomat,ymin,ymax,unit,axvline,axhline,confidence,text_font={"size":20,"family":"Arial","weight":"bold"},legend_font={"size":20,"family":"Arial","weight":"bold"}):
	'''plot the density dsitribution'''
	plt.rc("font",weight="bold")
	fig=plt.figure(figsize=(16,8))
	ax=fig.add_subplot(111)
	winLen=in_regionLengthParma+in_extendRegionLengthParma+1
	if len(samples) <=8:
		colors=["b","orangered","green","c","m","y","k","w"]
	else:
		colors=colors=sns.color_palette('husl',len(samples))

	if unit == 'codon':
		for i in np.arange(len(samples)):
			if Type=="start codon":
				plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,1],color=colors[i],label=samples[i],linewidth=1)
				if confidence:
					ax.fill_between(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,3],data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,4],color=colors[i],alpha=0.2)
				else:
					pass
				if axvline:
					ax.axvline(axvline,color="r",dashes=[1,2],clip_on=False,linewidth=2)
				else:
					pass

			else:
				plt.plot(np.arange(-winLen,0),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,2],color=colors[i],label=samples[i],linewidth=1)
				if confidence:
					ax.fill_between(np.arange(-winLen,0),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,5],data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,6],color=colors[i],alpha=0.2)
				else:
					pass

		ax.set_xlabel("Distance from "+Type + " (codon)",fontdict=text_font)
		ax.set_ylabel("Mean enrichment (A.U)",fontdict=text_font)
		if axhline:
			ax.axhline(axhline,color="r",dashes=[2,3],clip_on=False,linewidth=2)
		else:
			pass

		ax.spines["top"].set_visible(False)
		ax.spines["right"].set_visible(False)
		ax.spines["bottom"].set_linewidth(2)
		ax.spines["left"].set_linewidth(2)
		ax.tick_params(which="both",width=2,labelsize=20)
		if not ymin and not ymax:
			pass
		elif not ymin and ymax:
			ax.set_ylim(0,ymax)
		elif ymin and not ymax:
			raise IOError("Please offer the ymax parameter as well!")
		elif ymin and ymax:
			ax.set_ylim(ymin,ymax)
		else:
			raise IOError("Please enter correct ymin and ymax parameters!")
		plt.legend(loc="best",prop=legend_font)
		plt.tight_layout()
		plt.savefig(inOutPrefix+"_"+Type+"."+inOutFomat,format=inOutFomat)
		# plt.show()
		plt.close()
	elif unit =='nt':
		for i in np.arange(len(samples)):
			if Type=="start codon":
				plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,1],color=colors[i],label=samples[i],linewidth=1)
				if confidence:
					ax.fill_between(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,3],data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,4],color=colors[i],alpha=0.2)
				else:
					pass
				if axvline:
					ax.axvline(axvline,color="r",dashes=[1,2],clip_on=False,linewidth=2)
				else:
					pass

			else:
				plt.plot(np.arange(-winLen,0),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,2],color=colors[i],label=samples[i],linewidth=1)
				if confidence:
					ax.fill_between(np.arange(-winLen,0),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,5],data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,6],color=colors[i],alpha=0.2)
				else:
					pass
		ax.set_xlabel("Distance from "+Type + " (nt)",fontdict=text_font)
		ax.set_ylabel("Mean enrichment (A.U)",fontdict=text_font)
		if axhline:
			ax.axhline(axhline,color="r",dashes=[2,3],clip_on=False,linewidth=2)
		else:
			pass

		ax.spines["top"].set_visible(False)
		ax.spines["right"].set_visible(False)
		ax.spines["bottom"].set_linewidth(2)
		ax.spines["left"].set_linewidth(2)
		ax.tick_params(which="both",width=2,labelsize=20)
		if not ymin and not ymax:
			pass
		elif not ymin and ymax:
			ax.set_ylim(0,ymax)
		elif ymin and not ymax:
			raise IOError("Please offer the ymax parameter as well!")
		elif ymin and ymax:
			ax.set_ylim(ymin,ymax)
		else:
			raise IOError("Please enter correct ymin and ymax parameters!")
		plt.legend(loc="best",prop=legend_font)
		plt.tight_layout()
		plt.savefig(inOutPrefix+"_"+Type+"."+inOutFomat,format=inOutFomat)
		# plt.show()
		plt.close()
	else:
		raise IOError("Please reset your --unit parameter [codon or nt]")

def plot_density_for_each_sample(data,samples,Type,in_regionLengthParma,in_extendRegionLengthParma,inOutPrefix,inOutFomat,ymin,ymax,unit,axvline,axhline,confidence,text_font={"size":20,"family":"Arial","weight":"bold"},legend_font={"size":20,"family":"Arial","weight":"bold"}):
	'''plot the density dsitribution'''
	plt.rc("font",weight="bold")
	winLen=in_regionLengthParma+in_extendRegionLengthParma+1
	if len(samples) <=8:
		colors=["b","orangered","green","c","m","y","k","w"]
	else:
		colors=colors=sns.color_palette('husl',len(samples))
	sample_dict=dict([i for i in enumerate(samples)])
	sample_dict={j:i for i,j in sample_dict.items()}
	if unit == 'codon':
		for i in sample_dict.keys():
			fig=plt.figure(figsize=(16,8))
			ax=fig.add_subplot(111)
			if Type=="start codon":
				plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==i)].iloc[:,1],color=colors[sample_dict[i]],label=i,linewidth=1)
				if confidence:
					ax.fill_between(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,3],data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,4],color=colors[i],alpha=0.2)
				else:
					pass
				if axvline:
					ax.axvline(axvline,color="r",dashes=[1,2],clip_on=False,linewidth=2)
				else:
					pass

			else:
				plt.plot(np.arange(-winLen,0),data.iloc[np.where(data.iloc[:,0]==i)].iloc[:,2],color=colors[sample_dict[i]],label=i,linewidth=1)
				if confidence:
					ax.fill_between(np.arange(-winLen,0),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,5],data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,6],color=colors[i],alpha=0.2)
				else:
					pass
			ax.set_xlabel("Distance from "+Type + " (codon)",fontdict=text_font)
			ax.set_ylabel("Mean enrichment (A.U)",fontdict=text_font)
			if axhline:
				ax.axhline(1,color="r",dashes=[2,3],clip_on=False,linewidth=2)
			else:
				pass
			ax.spines["top"].set_visible(False)
			ax.spines["right"].set_visible(False)
			ax.spines["bottom"].set_linewidth(2)
			ax.spines["left"].set_linewidth(2)
			ax.tick_params(which="both",width=2,labelsize=20)
			if not ymin and not ymax:
				pass
			elif not ymin and ymax:
				ax.set_ylim(0,ymax)
			elif ymin and not ymax:
				raise IOError("Please offer the ymax parameter as well!")
			elif ymin and ymax:
				ax.set_ylim(ymin,ymax)
			else:
				raise IOError("Please enter correct ymin and ymax parameters!")
			plt.legend(loc="best",prop=legend_font)
			plt.tight_layout()
			plt.savefig(inOutPrefix+"_"+i+"_"+Type+"."+inOutFomat,format=inOutFomat)
			# plt.show()
			plt.close()

	elif unit =='nt':
		for i in sample_dict.keys():
			fig=plt.figure(figsize=(16,8))
			ax=fig.add_subplot(111)
			if Type=="start codon":
				plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==i)].iloc[:,1],color=colors[sample_dict[i]],label=i,linewidth=1)
				if confidence:
					ax.fill_between(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,3],data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,4],color=colors[i],alpha=0.2)
				else:
					pass
				if axvline:
					ax.axvline(axvline,color="r",dashes=[1,2],clip_on=False,linewidth=2)
				else:
					pass

			else:
				plt.plot(np.arange(-winLen,0),data.iloc[np.where(data.iloc[:,0]==i)].iloc[:,2],color=colors[sample_dict[i]],label=i,linewidth=1)
				if confidence:
					ax.fill_between(np.arange(-winLen,0),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,5],data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,6],color=colors[i],alpha=0.2)
				else:
					pass
			ax.set_xlabel("Distance from "+Type + " (nt)",fontdict=text_font)
			ax.set_ylabel("Mean enrichment (A.U)",fontdict=text_font)
			if axhline:
				ax.axhline(1,color="r",dashes=[2,3],clip_on=False,linewidth=2)
			else:
				pass
			ax.spines["top"].set_visible(False)
			ax.spines["right"].set_visible(False)
			ax.spines["bottom"].set_linewidth(2)
			ax.spines["left"].set_linewidth(2)
			ax.tick_params(which="both",width=2,labelsize=20)
			if not ymin and not ymax:
				pass
			elif not ymin and ymax:
				ax.set_ylim(0,ymax)
			elif ymin and not ymax:
				raise IOError("Please offer the ymax parameter as well!")
			elif ymin and ymax:
				ax.set_ylim(ymin,ymax)
			else:
				raise IOError("Please enter correct ymin and ymax parameters!")
			plt.legend(loc="best",prop=legend_font)
			plt.tight_layout()
			plt.savefig(i+"_"+Type+"."+inOutFomat,format=inOutFomat)
			# plt.show()
			plt.close()
	else:
		raise IOError("Please reset your --unit parameter [codon or nt]")

def change_motif_names(data,type_list):
		samples=data.iloc[:,0]
		if len(type_list) < 1:
			pass
		if len(type_list)==1:
			new_sample=pd.DataFrame([type_list[0]]*len(samples),columns=[samples.name])
		if len(type_list) > 1:
			new_sample=list(reduce(chain,[[i]*(len(samples)//len(type_list)) for i in type_list]))
			new_sample=pd.DataFrame(new_sample,columns=[samples.name])
		data.iloc[:,0]=new_sample
		return data

def main():
	parsed=create_parser()
	(options,args)=parsed.parse_args()
	(data,in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,ymin,ymax,label_list,unit,axvline,axhline,mode,start,window,step,slideWindow,confidence,)=(options.density_file,options.downstream_codon,options.upstream_codon,
	options.output_prefix,options.output_format,options.ymin,options.ymax,options.label_list,options.unit,options.axvline,options.axhline,options.mode,options.start_position,options.window,options.step,options.slideWindow,options.CI)
	if window%2 == 0:
		raise IOError("Please reset your --window parameter. It must be a odd number.")
	if (start-1) < (window-1)/2:
		raise IOError("Please reset your --step and --window parameters. The (window-1)/2 must be less than start-1")
	print("your input file is: "+str(data),file=sys.stderr)
	data=pd.read_csv(data,sep="\t")
	samples=np.unique(data.iloc[:,0])
	if label_list:
		label_list=label_list.strip().split(',')
		data=change_motif_names(data,label_list)
		samples=np.unique(data.iloc[:,0])
	else:
		pass
	## plot density
	text_font={"size":40,"family":"Arial","weight":"bold"}
	legend_font={"size":30,"family":"Arial","weight":"bold"}
	if not slideWindow:
		pass
	else:
		data=slide_window_average(data,samples,in_regionLengthParma,in_extendRegionLengthParma,output_prefix,start,window,step)
	if mode == 'all':
		plot_density_for_all_samples(data,samples,"start codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,ymin,ymax,unit,axvline,axhline,confidence,text_font=text_font,legend_font=legend_font)
		plot_density_for_all_samples(data,samples,"stop codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,ymin,ymax,unit,axvline,axhline,confidence,text_font=text_font,legend_font=legend_font)
	elif mode == 'single':
		plot_density_for_each_sample(data,samples,"start codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,ymin,ymax,unit,axvline,axhline,confidence,text_font=text_font,legend_font=legend_font)
		plot_density_for_each_sample(data,samples,"stop codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,ymin,ymax,unit,axvline,axhline,confidence,text_font=text_font,legend_font=legend_font)
	else:
		raise IOError("Please reset your --mode parameter.[all or single]")

	print("finished plot the ribosome footprint density",file=sys.stderr)

if __name__ =="__main__":
	main()

